import uuid
from typing import Optional

from fastapi import APIRouter, Depends

from src.api.dummy_db.schema import (
    UpdateDummyModel,
    GetDummyResponse,
)
{% if cookiecutter.add_elastic_search == "True" -%}
from src.api.dummy_db.schema import DummyElasticResponse, ElasticAdd
from src.services.elastic.schema import ElasticFilterModel
{%else%}
from src.api.dummy_db.schema import BaseDummyModel
{% endif %}

from src.models import DummyDBModel
from src.services.db.session import Session, db_session

router = APIRouter()
URL_PREFIX = "/dummy_db_obj"


@router.put("/")
async def create_dummy(dummy_obj: {% if cookiecutter.add_elastic_search == "True" -%}ElasticAdd{% else %}BaseDummyModel{% endif %}, session: Session = Depends(db_session)) -> None:
    """
    Add dummy object in database.
    If you have elastic search feature enabled it will be added in your index.
    """
    {% if cookiecutter.add_elastic_search == "True" -%}
    insert_query = DummyDBModel.create(**dummy_obj.dict()).returning(DummyDBModel.id)
    model_id = await session.scalar(insert_query)
    await DummyDBModel.elastic_add(
        model_id=model_id,
        **dummy_obj.dict()
    )
    {% else %}
    await session.execute(DummyDBModel.create(**dummy_obj.dict()))
    {% endif %}

@router.post("/{dummy_id}")
async def update_dummy_model(
        dummy_id: uuid.UUID,
        new_values: UpdateDummyModel,
        session: Session = Depends(db_session)
) -> None:
    await session.execute(DummyDBModel.update(dummy_id, **new_values.dict()))
    {% if cookiecutter.add_elastic_search == "True" -%}
    await DummyDBModel.elastic_update(
        model_id=dummy_id,
        **new_values.dict(exclude_unset=True)
    )
    {% endif %}


@router.delete("/{dummy_id}")
async def delete_dummy_model(dummy_id: uuid.UUID, session: Session = Depends(db_session)) -> None:
    await session.execute(DummyDBModel.delete(dummy_id))
    {% if cookiecutter.add_elastic_search == "True" -%}
    await DummyDBModel.elastic_delete(model_id=dummy_id)
    {% endif %}


@router.get("/", response_model=GetDummyResponse)
async def filter_dummy_models(
        dummy_id: Optional[uuid.UUID] = None,
        name: Optional[str] = None,
        surname: Optional[str] = None,
        session: Session = Depends(db_session)
) -> GetDummyResponse:
    filter_query = DummyDBModel.filter(
        dummy_id=dummy_id,
        name=name,
        surname=surname
    )
    results = await session.fetchall(filter_query)
    return GetDummyResponse(
        results=results
    )

{% if cookiecutter.add_elastic_search == "True" -%}
@router.get("/elastic", response_model=DummyElasticResponse)
async def dummy_elastic_filter(query: ElasticFilterModel = Depends(ElasticFilterModel)) -> DummyElasticResponse:
    results = await DummyDBModel.elastic_filter(**query.dict())
    return DummyElasticResponse(
        results=results
    )
{% endif %}



from typing import Optional

from aiopg.sa import Engine, create_engine
from sqlalchemy.engine.url import URL, make_url

from src.settings import settings

db_url = make_url(
    str(URL(
        drivername=settings.db_driver,
        username=settings.postgres_user,
        password=settings.postgres_password,
        host=settings.postgres_host,
        port=settings.postgres_port,
        database=settings.postgres_db,
    ))
)


class DBEngine:
    def __init__(self, connection_url: str) -> None:
        self.dsn = connection_url
        self.engine: Optional[Engine] = None

    async def connect(self) -> None:
        self.engine = await create_engine(dsn=self.dsn)

    @property
    def client(self) -> Engine:
        if self.engine is None:
            raise ValueError("Not connected to database")
        return self.engine

    async def close(self) -> None:
        if self.engine:
            self.engine.close()
            await self.engine.wait_closed()
        raise Exception("Not connected to database")


db_engine = DBEngine(str(db_url))



import os
import sys
from logging.config import fileConfig

from alembic import context
from sqlalchemy import create_engine
from sqlalchemy.engine.url import URL, make_url

sys.path.insert(0, os.path.abspath(""))
# this is the Alembic Config object, which provides
# access to the values within the .ini file in use.
# noqa
from src.settings import settings

config = context.config

# Interpret the config file for Python logging.
# This line sets up loggers basically.
fileConfig(config.config_file_name)

# add your model's MetaData object here
# for 'autogenerate' support
# from myapp import mymodel
# target_metadata = mymodel.Base.metadata
# noqa
from src.services.db import db_meta as target_metadata
from src.models import *  # isort:skip


def run_migrations_offline():
    """Run migrations in 'offline' mode.

    This configures the context with just a URL
    and not an Engine, though an Engine is acceptable
    here as well.  By skipping the Engine creation
    we don't even need a DBAPI to be available.

    Calls to context.execute() here emit the given string to the
    script output.

    """
    url = make_url(
        URL(
            drivername=settings.db_driver,
            username=settings.postgres_user,
            password=settings.postgres_password,
            host=settings.postgres_host,
            port=settings.postgres_port,
            database=settings.postgres_db,
        )
    )

    context.configure(
        url=url,
        target_metadata=target_metadata,
        literal_binds=True,
        dialect_opts={"paramstyle": "named"},
    )

    with context.begin_transaction():
        context.run_migrations()


def run_migrations_online():
    """Run migrations in 'online' mode.

    In this scenario we need to create an Engine
    and associate a connection with the context.

    """
    url = make_url(
        URL(
            drivername=settings.db_driver,
            username=settings.postgres_user,
            password=settings.postgres_password,
            host=settings.postgres_host,
            port=settings.postgres_port,
            database=settings.postgres_db,
        )
    )

    connectable = create_engine(
        str(url),
    )

    with connectable.connect() as connection:
        context.configure(connection=connection, target_metadata=target_metadata)

        with context.begin_transaction():
            context.run_migrations()


if context.is_offline_mode():
    run_migrations_offline()
else:
    run_migrations_online()



"""
Pre generation hooks
Currently only synchronizes passwords


{{ cookiecutter.update({"postgres_password": random_ascii_string(20) }) }}
{{ cookiecutter.update({"redis_password": random_ascii_string(20) }) }}
"""


from fastapi import HTTPException
from httpx import Response


class ServiceError(HTTPException):
    """
    This exception can be returned from handler as is.
    If this exception is raised you can skip handling.

    Your application will return normal response
     with status_code=400 and human-readable message.
    """
    def __init__(self, action: str, service_response: Response) -> None:
        try:
            error_data = service_response.json()
        except LookupError:
            error_data = service_response.text
        self.detail = (
            f"Can't {action}. Getter respond: {error_data}"
            f" (status code: {service_response.status_code})"
        )
        self.response = service_response
        self.status_code = 400



from typing import Optional, Tuple, Union

import aioredis
from aioredis import Redis
from loguru import logger
from src.settings import settings


class RedisClient:
    """
    Super simple redis client.
    """
    def __init__(self, address: Union[str, Tuple[str, int]], password: str):
        self.address = address
        self.password = password
        self._client: Optional[Redis] = None

    async def create_pool(self) -> None:
        logger.debug("Creating redis pool")
        self._client = await aioredis.create_redis_pool(
            self.address, password=self.password
        )
        logger.debug("Redis pool created")

    @property
    def client(self) -> Redis:
        """
        Get actual Redis client.
        :return:
        """
        if self._client is None:
            raise Exception("Not connected to redis")
        return self._client

    async def shutdown(self) -> None:
        """
        Shutdown session for redis.
        Close redis pool.
        """
        logger.debug("Shutting down redis")
        if self._client is None:
            return
        self._client.close()
        await self._client.wait_closed()
        logger.debug("Redis connection pool closed")


redis = RedisClient(
    address=(settings.redis_host, settings.redis_port), password=settings.redis_password
)



from fastapi import APIRouter

from src.api.httpbin import URL_PREFIX as httpbin_prefix
from src.api.httpbin import router as httpbin_router
{% if cookiecutter.add_redis == "True" -%}
from src.api.redis_api import URL_PREFIX as redis_prefix
from src.api.redis_api import router as redis_router
{% endif %}
{% if cookiecutter.add_dummy_model == "True" -%}
from src.api.dummy_db import URL_PREFIX as dummy_db_prefix
from src.api.dummy_db import router as dummy_db_router
{% endif %}

api_router = APIRouter()

api_router.include_router(
    router=httpbin_router, prefix=httpbin_prefix, tags=['HTTPBin']
)
{% if cookiecutter.add_redis == "True" -%}
api_router.include_router(
    router=redis_router, prefix=redis_prefix, tags=["Redis"]
)
{% endif %}
{% if cookiecutter.add_dummy_model == "True" -%}
api_router.include_router(
    router=dummy_db_router, prefix=dummy_db_prefix, tags=["Dummy db object"]
)
{% endif %}


import argparse
from typing import Any, Dict, Optional

from fastapi import FastAPI
from gunicorn.app.base import BaseApplication

from src.server import app


class StandaloneApplication(BaseApplication):
    def __init__(self, application_instance: FastAPI, run_options: Optional[Dict[str, Any]] = None):
        self.options = run_options or {}
        self.application = application_instance
        super().__init__()

    def load_config(self) -> None:
        config = {
            key: value
            for key, value in self.options.items()
            if key in self.cfg.settings and value is not None
        }
        for key, value in config.items():
            self.cfg.set(key.lower(), value)

    def load(self) -> FastAPI:
        return self.application


def parse_args() -> argparse.Namespace:
    parser = argparse.ArgumentParser()
    parser.add_argument("--host", type=str, default="0.0.0.0")
    parser.add_argument("--port", type=int, default=8000)
    parser.add_argument("--pid-file", type=str, default="/tmp/{{cookiecutter.project_name}}.pid")
    return parser.parse_args()


if __name__ == "__main__":
    args = parse_args()
    options = {
        "bind": f"{args.host}:{args.port}",
        "workers": 4,
        "worker_class": "uvicorn.workers.UvicornWorker",
        "pidfile": args.pid_file,
    }
    StandaloneApplication(app, options).run()



import uuid
from datetime import datetime
from typing import Optional, List
from src.models.dummy_db_model import DummyElasticFilter
from pydantic import Field, BaseConfig
from pydantic.main import BaseModel


class BaseDummyModel(BaseModel):
    name: str = Field(example="Dummy name")
    surname: str = Field(example="Dummy surname")


class ReturnDummyModel(BaseDummyModel):
    id: uuid.UUID
    created_at: datetime
    updated_at: datetime

    class Config(BaseConfig):
        orm_mode = True


class GetDummyResponse(BaseModel):
    results: List[ReturnDummyModel]

{% if cookiecutter.add_elastic_search == "True" -%}
class DummyElasticResponse(BaseModel):
    results: List[DummyElasticFilter]

class ElasticAdd(BaseDummyModel):
    tags: str = Field(default="")
{% endif %}

class UpdateDummyModel(BaseModel):
    name: Optional[str] = Field(default=None, example="New name")
    surname: Optional[str] = Field(default=None, example="New surname")
    {% if cookiecutter.add_elastic_search == "True" -%}
    tags: Optional[str] = Field(default=None, example="tag1,tag2")
    {% endif %}

class DummyFiltersModel(BaseModel):
    dummy_id: Optional[uuid.UUID] = Field(default=None)
    name: Optional[str] = Field(default=None)
    surname: Optional[str] = Field(default=None)



#!/usr/bin/env python
import json
import os
import shutil
from argparse import Namespace

import pre_commit.constants as pre_commit_constants
import pre_commit.main as pre_commit
from pygit2 import init_repository
from termcolor import cprint

MANIFEST = "conditional_files.json"


def delete_resource(resource):
    if os.path.isfile(resource):
        print("removing file: {}".format(resource))
        os.remove(resource)
    elif os.path.isdir(resource):
        print("removing directory: {}".format(resource))
        shutil.rmtree(resource)


def delete_resources_for_disabled_features():
    with open(MANIFEST) as manifest_file:
        manifest = json.load(manifest_file)
        for feature in manifest['features']:
            if not feature['enabled'] == "true":
                print("removing resources for disabled feature {}...".format(feature['name']))
                for resource in feature['resources']:
                    delete_resource(resource)
    print("cleanup complete, removing manifest...")
    delete_resource(MANIFEST)


def init_repo():
    store = pre_commit.Store()
    repo_path = os.getcwd()
    repo = init_repository(repo_path)
    cprint("Git repository initialized.", "green")
    repo.index.add_all()
    repo.index.write()
    cprint("Added files to index.", "green")
    pre_commit.install(
        config_file=pre_commit_constants.CONFIG_FILE,
        store=store,
        hook_types=["pre-commit"],
        overwrite=False
    )
    cprint("pre-commit installed.", "green")
    run_namespace = Namespace(
        all_files=True,
        files=[],
        hook_stage='commit',
        from_ref=None,
        to_ref=None,
        remote_name=None,
        checkout_type=None,
        hook=None,
        verbose=False,
        color=True,
        show_diff_on_failure=False,
        is_squash_merge=False,
    )
    pre_commit.run(
        config_file=pre_commit_constants.CONFIG_FILE,
        store=store,
        args=run_namespace
    )
    repo.index.add_all()
    repo.index.write()


if __name__ == "__main__":
    delete_resources_for_disabled_features()
    init_repo()



"""Added dummy model

Revision ID: f9c5d2dba936
Revises:
Create Date: 2020-10-05 23:56:58.658606

"""
import sqlalchemy as sa
from alembic import op
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = 'f9c5d2dba936'
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('dummydbmodel',
                    sa.Column('id', postgresql.UUID(as_uuid=True), nullable=False),
                    sa.Column('name', sa.String(), nullable=False),
                    sa.Column('surname', sa.String(), nullable=False),
                    {% if cookiecutter.add_elastic_search == "True" -%}sa.Column('tags', sa.String(), nullable=False, default=""),{% endif %}
                    sa.Column('created_at', sa.DateTime(timezone=True), server_default=sa.text('clock_timestamp()'),
                              nullable=False),
                    sa.Column('updated_at', sa.DateTime(timezone=True), server_default=sa.text('clock_timestamp()'),
                              nullable=False),
                    sa.PrimaryKeyConstraint('id')
                    )
    op.create_index(op.f('ix_dummydbmodel_name'), 'dummydbmodel', ['name'], unique=False)
    op.create_index(op.f('ix_dummydbmodel_surname'), 'dummydbmodel', ['surname'], unique=False)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(op.f('ix_dummydbmodel_surname'), table_name='dummydbmodel')
    op.drop_index(op.f('ix_dummydbmodel_name'), table_name='dummydbmodel')
    op.drop_table('dummydbmodel')
    # ### end Alembic commands ###



from fastapi import APIRouter

from src.services.httpbin.client import http_bin_service
from src.services.httpbin.schema import DummyHttpBinObj, HTTPBinResponse

router = APIRouter()
URL_PREFIX = "/http_bin"


@router.put("/", response_model=HTTPBinResponse)
async def create_dummy_obj(dummy_obj: DummyHttpBinObj) -> HTTPBinResponse:
    return await http_bin_service.put_req(dummy_obj)


@router.delete("/{obj_id}", response_model=HTTPBinResponse)
async def delete_dummy_obj(obj_id: str) -> HTTPBinResponse:
    return await http_bin_service.delete_req(obj_id)



import pytest
import sqlalchemy as sa
from sqlalchemy.engine import Connection
from starlette.testclient import TestClient

from src.models import DummyDBModel
from tests import TestSettings

test_data = [
    TestSettings(
        request_data=dict(
            dummy_id="b4439fc8-8fde-4079-ab2c-d61e46e22f25",
            json=dict(
                name="test dummy object",
                surname="Cesar"
            ),
        ),
    ),
]


@pytest.mark.parametrize("test_conf", test_data)
def test_delete_dummy_obj(
        pg_conn: Connection, test_conf: TestSettings, app_fixture: TestClient
) -> None:
    with app_fixture as client:
        put_result = client.put(
            "/dummy_db_obj/",
            json=test_conf.request_data["json"],
        )
        assert put_result.status_code == 200
        with pg_conn.begin():
            result = pg_conn.execute(DummyDBModel.select_query(DummyDBModel.id)).first()
            assert len(result) == 1
        result = client.delete(f"/dummy_db_obj/{result[0]}")
        assert result.status_code == 200
        with pg_conn.begin():
            result = pg_conn.execute(sa.func.count(DummyDBModel.id)).first()
            assert result[0] == 0



import warnings
from typing import Generator

import alembic.config
import pytest
from fastapi.testclient import TestClient
from sqlalchemy import create_engine
from sqlalchemy.engine import Connection, Engine
from sqlalchemy.engine.url import URL, make_url
from sqlalchemy.exc import ProgrammingError

from src.services.db import db_url
from src.settings import settings

warnings.filterwarnings("ignore", category=DeprecationWarning)


def get_engine() -> Engine:
    pg_db_url = make_url(
        str(URL(
            drivername=settings.db_driver,
            username=settings.postgres_user,
            password=settings.postgres_password,
            host=settings.postgres_host,
            port=settings.postgres_port,
            database="postgres",
        ))
    )
    engine = create_engine(str(pg_db_url))
    return engine


def run_psql_without_transaction(command: str) -> None:
    engine = get_engine()
    connection = engine.connect()
    connection.connection.set_isolation_level(0)
    connection.execute(command)
    connection.connection.set_isolation_level(1)
    connection.close()


@pytest.fixture(scope="session")
def create_database() -> None:
    try:
        run_psql_without_transaction(f"CREATE DATABASE {settings.postgres_db}")
    except ProgrammingError:
        pass


def prepare_db() -> None:
    alembic.config.main(
        [
            "--raiseerr",
            "upgrade",
            "head",
        ]
    )


def drop_db() -> None:
    alembic.config.main(
        [
            "--raiseerr",
            "downgrade",
            "base",
        ]
    )


@pytest.fixture(scope="function")
def create_db(create_database: None) -> Generator[None, None, None]:
    prepare_db()
    yield
    drop_db()


@pytest.fixture(scope="function")
def app_fixture(create_db: None) -> TestClient:
    from src.server import app

    api_client = TestClient(app)

    return api_client


@pytest.fixture(scope="function")
def pg_conn() -> Generator[Connection, None, None]:
    engine = create_engine(str(db_url), pool_size=0, echo=True)
    conn = engine.connect()

    yield conn

    conn.close()



import uuid
from typing import Any, Dict, Optional, Tuple, Type, Union

import sqlalchemy as sa
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.ext.declarative import as_declarative, declared_attr
from sqlalchemy.orm.attributes import InstrumentedAttribute
from src.services.db.db_meta import meta


@as_declarative(metadata=meta)
class Base:
    """
    Base class for all models

    It has some very cool methods which allows you
    to ship autocompletion and type verification to SQLAlchemy models.

    >>> class Model(Base):
    >>>     name = sa.Column(sa.String())
    >>>
    >>>     @classmethod
    >>>     def get_by_name(cls, name: str) -> sa.sql.Select:
    >>>         return Model.select_query(cls.id).where(cls.name == name)
    ...
    >>> session.fetchall(Model.get_by_name("random_name"))

    `id`, `created_at` and `updated_at` columns are created
    automatically for all models.

    Basic settings form models such as `__name__`, `__table__` and `__table_args__`
    defined with types to allow mypy verify this data.

    `__tablename__` generated automatically based on class name.
    """

    __name__: str
    __table__: sa.Table
    __table_args__: Tuple[Any, ...]

    @declared_attr
    def __tablename__(self) -> str:
        return self.__name__.lower()

    @declared_attr
    def id(self) -> Any:
        return sa.Column(
            UUID(as_uuid=True),
            primary_key=True,
            default=uuid.uuid4
        )

    @declared_attr
    def created_at(self) -> Any:
        return sa.Column(
            sa.DateTime(timezone=True),
            server_default=sa.text("clock_timestamp()"),
            nullable=False,
        )

    @declared_attr
    def updated_at(self) -> Any:
        return sa.Column(
            sa.DateTime(timezone=True),
            server_default=sa.text("clock_timestamp()"),
            onupdate=sa.text("clock_timestamp()"),
            nullable=False,
        )

    @classmethod
    def get(
        cls, pk: Union[uuid.UUID, str], *fields: InstrumentedAttribute
    ) -> Optional[Any]:
        return cls.select_query(*fields).where(cls.id == pk)

    @classmethod
    def exists(cls, pk: Union[uuid.UUID, str]) -> sa.sql.expression.Exists:
        return sa.exists().where(cls.id == pk)

    @classmethod
    def delete(cls, pk: uuid.UUID) -> sa.sql.Delete:
        return cls.delete_query().where(cls.id == pk)

    @classmethod
    def select_query(
        cls,
        *columns: Union[InstrumentedAttribute, Type["Base"]],
        use_labels: bool = False,
    ) -> sa.sql.Select:
        return sa.select(columns or [cls], use_labels=use_labels)

    @classmethod
    def insert_query(cls, **values: Any) -> sa.sql.Insert:
        return cls.__table__.insert().values(**values)

    @classmethod
    def update_query(cls) -> sa.sql.Update:
        return cls.__table__.update()

    @classmethod
    def delete_query(cls) -> sa.sql.Delete:
        return cls.__table__.delete()

    def as_dict(self) -> Dict[str, Any]:
        return {c.name: getattr(self, c.key) for c in self.__table__.columns}



import asyncio

import aioschedule as schedule
from loguru import logger

from src.settings import settings


async def scheduled_task() -> None:
    logger.info("I was scheduled to run now")


schedule.every(settings.schedule_timer).seconds.do(scheduled_task)

if __name__ == "__main__":
    logger.debug("Started scheduler")
    loop = asyncio.get_event_loop()
    while True:
        loop.run_until_complete(schedule.run_pending())


import uuid
from typing import TypeVar, List, Optional, Any, Generic, Union, Dict

from elasticsearch import NotFoundError
from elasticsearch_dsl import Search
from elasticsearch_dsl.query import MultiMatch
from loguru import logger

from src.services.elastic.client import elastic_client
from src.services.elastic.schema import ESReturnModel

SearchModel = TypeVar("SearchModel", covariant=True, bound=ESReturnModel)


class ElasticModelMixin(Generic[SearchModel]):
    """
    Mixin to simplify ElasticSearch integration with SQLAlchemy models.

    It has three main variables to override in model.

    1) __es_index_name - it's a name of the index in ElasticSearch where
        data for model would be stored.
    2) __es_search_fields - the list of fields use in search query.
    3) __es_search_type - covariant of `SearchModel` to return from `elastic_filter`.
        If you won't define it `elastic_filter` will return dict.
    """
    __es_index_name: str = "default_index"
    __es_search_fields: List[str] = []
    __es_search_type: Optional[SearchModel] = None

    @classmethod
    async def elastic_add(
        cls, model_id: uuid.UUID, *, tags: Optional[str] = "", **kwargs: Any
    ) -> None:
        """
        Add object to elastic index.

        >>> await Model.elastic_index(uuid.uuid4(), name="La", surname="Tex")
        """
        ret = await elastic_client.index(
            index=cls.__es_index_name,
            body={"tags": tags.split(",") if tags else [], **kwargs},
            id=str(model_id),
        )
        result = ret["result"]
        logger.debug(f"{result} '{cls.__name__}:{model_id}' in elastic")

    @classmethod
    async def elastic_filter(
        cls, *, query: str, offset: int, limit: int
    ) -> List[Union[SearchModel, Dict[str, Any]]]:
        """
        Filter existing models in ElasticSearch by string query.
        This function uses __es_search_fields as fields for phrase_prefix query.

        >>> await Model.elastic_filter(query="La")
        """
        elastic_query = Search()
        if query:
            elastic_query = elastic_query.query(
                MultiMatch(
                    type="phrase_prefix", query=query, fields=cls.__es_search_fields
                )
            )
        elastic_query = elastic_query[offset : offset + limit]
        search_res = await elastic_client.search(elastic_query.to_dict())
        hits = search_res.get("hits", {}).get("hits", [])
        results = []
        constructor = cls.__es_search_type or dict
        for hit in hits:
            logger.debug(hit)
            results.append(constructor(id=hit.get("_id"), **hit.get("_source", {})))
        return results

    @classmethod
    async def elastic_update(
        cls,
        model_id: uuid.UUID,
        **body: Any,
    ) -> None:
        """
        Update model in ElasticSearch by id.
        """
        tags = body.get("tags", None)
        if tags is not None:
            body["tags"] = tags.split(",") if tags else []
        logger.debug(f"Updating '{cls.__name__}:{model_id}' in ES")
        try:
            await elastic_client.update(
                index=cls.__es_index_name,
                id=str(model_id),
                body={"doc": body},
                refresh=True,
            )
        except NotFoundError:
            logger.debug(f"Can't update unknown {cls.__name__} in es")

    @classmethod
    async def elastic_delete(cls, model_id: uuid.UUID) -> None:
        """
        Delete object from ElasticSearch index.
        """
        try:
            await elastic_client.delete(index=cls.__es_index_name, id=str(model_id))
        except NotFoundError:
            logger.debug(f"'{cls.__name__}:{model_id}' was not found in ES")

    @classmethod
    async def elastic_create_index(cls) -> None:
        """
        Create index if it's not exists.
        """
        if not await elastic_client.indices.exists(cls.__es_index_name):
            logger.debug(f"Creating elastic index {cls.__es_index_name}")
            await elastic_client.indices.create(cls.__es_index_name)



from enum import Enum

from pydantic import BaseSettings, Field


class AppMode(Enum):
    DEV = "development"
    PROD = "production"


class Settings(BaseSettings):
    # Application settings
    fastapi_env: AppMode = Field(default=AppMode.PROD)
    log_level: str = Field(default="INFO")
    # Auxiliary database settings
    db_echo: bool = Field(default=False)
    db_driver: str = Field(default="postgresql")
    # Postgres connection settings
    postgres_db: str = Field(...)
    postgres_host: str = Field(...)
    postgres_port: str = Field(...)
    postgres_user: str = Field(...)
    postgres_password: str = Field(...)
    {% if cookiecutter.add_redis == "True" -%}
    # Redis connection settings
    redis_host: str = Field(...)
    redis_port: int = Field(default=6379)
    redis_password: str = Field(...)
    {% endif %}
    {% if cookiecutter.add_scheduler == "True" -%}
    schedule_timer: int = Field(default=20)
    {% endif %}
    # httpbin client settings
    httpbin_host: str = Field(default="https://httpbin.org/")
    {% if cookiecutter.add_elastic_search == "True" -%}
    elastic_host: str = Field(...)
    {% endif %}

    @property
    def is_dev(self) -> bool:
        return self.fastapi_env == AppMode.DEV


settings = Settings()


import uuid
from typing import Optional, List

from pydantic import Field, BaseModel


class ESReturnModel(BaseModel):
    id: uuid.UUID
    tags: Optional[List[str]]


class ElasticFilterModel(BaseModel):
    query: str = Field(default="")
    limit: int = Field(default=100, lt=400)
    offset: int = Field(default=0)



from typing import Any, List, AsyncGenerator

from aiopg.sa import SAConnection

from src.services.db import db_engine


class Session:
    """
    Database session object.
    It used to acquire connection from pool
    and execute many queries using one connection.
    """
    def __init__(self, connection: SAConnection):
        self.connection = connection

    async def execute(self, query: Any) -> Any:
        """
        Simply execute SQLAlchemy generated query
        or string query and return raw results.
        """
        return await self.connection.execute(query)

    async def fetchone(self, query: Any) -> Any:
        """
        Get one object from database by query.
        """
        cursor = await self.connection.execute(query)
        return await cursor.fetchone()

    async def scalar(self, query: Any) -> Any:
        """
        Scalar returns first column of the first result.
        It's convenient fot such things as getting function result.

        >>> session = Session(connection)
        >>> session.scalar(Model.insert_query(**values).returning(Model.id))
        UUID('7a9ffbfe-f871-42fb-a371-7bd29227a9ff')
        """
        result = await self.fetchone(query)
        return result[0]

    async def fetchall(self, query: Any) -> List[Any]:
        """
        Get all matching objects from database by query.
        """
        cursor = await self.connection.execute(query)
        return await cursor.fetchall()


async def db_session() -> AsyncGenerator[Session, None]:
    """
    Dependency to acquire connection from database pool
    and close it when handler function is proceeds.
    """
    connection = await db_engine.client.acquire()
    session = Session(connection)
    try:
        yield session
    finally:
        await connection.close()



from pathlib import Path

from cookiecutter.exceptions import FailedHookException, OutputDirExistsException
from cookiecutter.main import cookiecutter
from termcolor import cprint

script_dir = Path(__file__).parent


def main():
    try:
        cookiecutter(template=f"{script_dir}/template")
    except (FailedHookException, OutputDirExistsException) as exc:
        if isinstance(exc, OutputDirExistsException):
            cprint(
                "Directory with such name already exists!",
                "red"
            )
        return
    cprint("Project successfully generated. You can read information about usage in README.md")


if __name__ == "__main__":
    main()



from typing import Any, Dict, Optional

from pydantic import Field
from pydantic.main import BaseModel


class TestSettings(BaseModel):
    __test__ = False

    request_data: Dict[str, Any]
    response_code: Optional[int]
    response_json: Optional[Dict[Any, Any]] = Field(None)
    validation_data: Optional[Dict[Any, Any]] = Field(None)



from httpx import AsyncClient, Response
from loguru import logger

from src.exceptions import ServiceError
from src.services.httpbin.schema import DummyHttpBinObj, HTTPBinResponse
from src.settings import settings


class HttpBinService:
    def __init__(self, http_bin_host: str) -> None:
        self.host = http_bin_host

    @property
    def client(self) -> AsyncClient:
        """
        Generate a client with base_url
        :return: async http client
        """
        return AsyncClient(
            base_url=self.host
        )

    @classmethod
    def catch_response(cls, response: Response, action: str, *codes: int) -> None:
        logger.debug("Response got")
        logger.debug(response.url)
        logger.debug(response.status_code)
        logger.debug(response.headers)
        logger.debug(response.text)
        if response.status_code not in codes:
            raise ServiceError(action, response)

    async def put_req(self, dummy_obj: DummyHttpBinObj) -> HTTPBinResponse:
        """
        Create dummy object in HTTPBin API.
        :param dummy_obj: target to create
        :return: None if success, raise an error if status code is not 201
        """
        async with self.client as http:
            response = await http.put("/put", json=dummy_obj.dict())
            self.catch_response(response, "Create dummy object", 200)
        return HTTPBinResponse(**response.json())

    async def delete_req(self, dummy_id: str) -> HTTPBinResponse:
        """
        Delete dummy_obj from HTTPBin API.
        :param dummy_id: target to delete
        :return: None if success, raise an error if status code is not 200
        """
        async with self.client as http:
            response = await http.delete("/delete", params={
                "dummy_id": dummy_id
            })
            self.catch_response(response, "delete bucket", 200)
        return HTTPBinResponse(**response.json())


http_bin_service = HttpBinService(
    settings.httpbin_host
)



from fastapi import APIRouter, HTTPException

from src.api.redis_api.schema import RedisSetModel, RedisValueModel
from src.services.redis import redis

router = APIRouter()
URL_PREFIX = "/redis"


@router.put("/set")
async def set_redis_value(
        redis_target: RedisSetModel,
) -> None:
    set_key = await redis.client.set(redis_target.key, redis_target.value, expire=redis_target.expire)
    if not set_key:
        raise HTTPException(
            status_code=400,
            detail="No, you can't",
        )


@router.get("/get", response_model=RedisValueModel)
async def get_redis_value(
        key: str,
) -> RedisValueModel:
    value = await redis.client.get(key, encoding='utf-8')
    if not value:
        raise HTTPException(
            status_code=400,
            detail="I can't get it",
        )
    return RedisValueModel(value=value)



import uuid
from typing import Optional

from sqlalchemy import Column, String, sql
{% if cookiecutter.add_elastic_search == "True" -%}
from src.services.elastic import ElasticModelMixin
from src.services.elastic.schema import ESReturnModel
{% endif %}
from src.services.db import Base

{% if cookiecutter.add_elastic_search == "True" -%}
class DummyElasticFilter(ESReturnModel):
    name: str
    surname: str
{% endif %}

class DummyDBModel(Base{% if cookiecutter.add_elastic_search == "True" -%}, ElasticModelMixin[DummyElasticFilter]{% endif %}):
    name = Column(String, nullable=False, index=True)
    surname = Column(String, nullable=False, index=True)
    {% if cookiecutter.add_elastic_search == "True" -%}
    tags = Column(String, nullable=False, default="")

    __es_index_name = "dummy_elastic_index"
    __es_search_fields = ["name", "surname"]
    __es_search_type = DummyElasticFilter
    {% endif %}

    @classmethod
    def create(
            cls,
            *,
            name: str,
            surname: str,
            {% if cookiecutter.add_elastic_search == "True" -%}tags: str = "",{% endif %}
    ) -> sql.Insert:
        return cls.insert_query(
            name=name,
            surname=surname,
            {% if cookiecutter.add_elastic_search == "True" -%}tags=tags,{% endif %}
        )

    @classmethod
    def delete(cls, dummy_id: uuid.UUID) -> sql.Delete:
        return cls.delete_query().where(cls.id == dummy_id)

    @classmethod
    def update(cls,
               dummy_id: uuid.UUID,
               *,
               name: Optional[str] = None,
               surname: Optional[str] = None,
               {% if cookiecutter.add_elastic_search == "True" -%}tags: Optional[str] = None,{% endif %}
               ) -> sql.Update:
        new_values = {}
        if name:
            new_values[cls.name] = name
        if surname:
            new_values[cls.surname] = surname
        if tags is not None:
            new_values[cls.tags] = tags
        return cls.update_query().where(cls.id == dummy_id).values(new_values)

    @classmethod
    def filter(cls, *,
               dummy_id: Optional[uuid.UUID] = None,
               name: Optional[str] = None,
               surname: Optional[str] = None
               ) -> sql.Select:
        query = cls.select_query()
        if dummy_id:
            query = query.where(cls.id == dummy_id)
        if name:
            query = query.where(cls.name == name)
        if surname:
            query = query.where(cls.surname == surname)
        return query



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['fastapi_template',
 'fastapi_template.template.hooks',
 'fastapi_template.template.{{cookiecutter.project_name}}',
 'fastapi_template.template.{{cookiecutter.project_name}}.migrations',
 'fastapi_template.template.{{cookiecutter.project_name}}.migrations.versions',
 'fastapi_template.template.{{cookiecutter.project_name}}.src',
 'fastapi_template.template.{{cookiecutter.project_name}}.src.api',
 'fastapi_template.template.{{cookiecutter.project_name}}.src.api.dummy_db',
 'fastapi_template.template.{{cookiecutter.project_name}}.src.api.httpbin',
 'fastapi_template.template.{{cookiecutter.project_name}}.src.api.redis_api',
 'fastapi_template.template.{{cookiecutter.project_name}}.src.models',
 'fastapi_template.template.{{cookiecutter.project_name}}.src.services',
 'fastapi_template.template.{{cookiecutter.project_name}}.src.services.db',
 'fastapi_template.template.{{cookiecutter.project_name}}.src.services.elastic',
 'fastapi_template.template.{{cookiecutter.project_name}}.src.services.httpbin',
 'fastapi_template.template.{{cookiecutter.project_name}}.tests']

package_data = \
{'': ['*'],
 'fastapi_template': ['template/*'],
 'fastapi_template.template.{{cookiecutter.project_name}}': ['envs/*',
                                                             'systemd/*']}

install_requires = \
['cookiecutter>=1.7.2,<2.0.0',
 'pre-commit>=2.8.2,<3.0.0',
 'pygit2>=1.4.0,<2.0.0',
 'termcolor>=1.1.0,<2.0.0']

entry_points = \
{'console_scripts': ['fastapi_template = fastapi_template.main:main']}

setup_kwargs = {
    'name': 'fastapi-template',
    'version': '1.2.3',
    'description': 'Feature-rich robust FastAPI template',
    'long_description': '![python version](https://img.shields.io/pypi/pyversions/fastapi_template?style=flat-square) ![Build status](https://img.shields.io/github/workflow/status/s3rius/FastAPI-template/Release%20python%20package?style=flat-square) [![version](https://img.shields.io/pypi/v/fastapi_template?style=flat-square)](https://pypi.org/project/fastapi-template/)\n\n<div align="center">\n<img src="https://raw.githubusercontent.com/s3rius/FastAPI-template/master/images/logo.png" width=700>\n<div><i>Fast and flexible general-purpose template for your API.</i></div>\n</div>\n\n\n## Usage\n⚠️ [Git](https://git-scm.com/downloads), [Python](https://www.python.org/), and [Docker-compose](https://docs.docker.com/compose/install/) must be installed and accessible ⚠️\n\n```bash\npython3 -m pip install fastapi_template\nfastapi_template\n# Answer prompts questions\n# ???\n# 🍪 Enjoy your new project 🍪\ncd new_project\ndocker-compose up --build\n```\n\n## Features\nCurrently supported features:\n- redis\n- systemd units\n- Example (dummy) SQLAlchemy model\n- Elastic Search support\n- Scheduler support\n',
    'author': 'Pavel Kirilin',
    'author_email': 'win10@list.ru',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/s3rius/FastAPI-template',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from typing import Any, Dict, Optional

from pydantic import BaseModel, Field


class DummyHttpBinObj(BaseModel):
    id: int
    name: str
    surname: str


class HTTPBinResponse(BaseModel):
    args: Dict[str, Any]
    data: str
    files: Dict[str, Any]
    form: Dict[str, str]
    headers: Dict[str, str]
    json_data: Optional[Dict[str, Any]] = Field(alias="json")
    origin: str
    url: str



import sys
from typing import Any, Callable

import alembic.config
from fastapi import FastAPI
from loguru import logger
from starlette.requests import Request
from starlette.responses import UJSONResponse

from src.api import api_router
from src.services.db import db_engine
{% if cookiecutter.add_redis == "True" -%}
from src.services.redis import redis
{% endif %}
from src.settings import settings

logger.configure(
    handlers=[{
        "sink": sys.stderr,
        "backtrace": False,
        "diagnose": settings.is_dev,
        "catch": False,
        "colorize": settings.is_dev,
    }]
)

app = FastAPI(
    title="{{cookiecutter.project_name}}",
    description="{{cookiecutter.project_description}}",
    default_response_class=UJSONResponse
)

app.include_router(prefix="", router=api_router)


@app.on_event("startup")
async def startup() -> None:
    {% if cookiecutter.add_redis == "True" -%}
    await redis.create_pool()
    {% endif %}
    await db_engine.connect()


@app.on_event("shutdown")
async def shutdown() -> None:
    {% if cookiecutter.add_redis == "True" -%}
    await redis.shutdown()
    {% endif %}
    await db_engine.close()



@app.middleware("http")
async def exception_logger(request: Request, call_next: Callable[..., Any]) -> Any:
    """
    Just to log all exception with godlike logger from loguru.
    """
    target_func = logger.catch()(call_next)
    response = await target_func(request)
    return response


if settings.is_dev:
    alembic.config.main(
        [
            "--raiseerr",
            "upgrade",
            "head",
        ]
    )
